/*
 *  Author: bwilliams
 *  Created: April 6, 2012
 *
 *  Copyright (C) 2012-2016 VMware, Inc.  All rights reserved. -- VMware Confidential
 *
 *  This code was generated by the script "build/dev/codeGen/genCppXml". Please
 *  speak to Brian W. before modifying it by hand.
 *
 */

#ifndef PayloadEnvelopeXmlRoots_h_
#define PayloadEnvelopeXmlRoots_h_

#include "Doc/PayloadEnvelopeDoc/CPayloadEnvelopeDoc.h"

#include "Doc/DocXml/PayloadEnvelopeXml/PayloadEnvelopeXmlLink.h"
#include "Doc/DocXml/PayloadEnvelopeXml/PayloadEnvelopeXmlRoots.h"

namespace Caf {

	namespace XmlRoots {
		/// Saves the PayloadEnvelopeDoc to a string.
		std::string PAYLOADENVELOPEXML_LINKAGE savePayloadEnvelopeToString(
			const SmartPtrCPayloadEnvelopeDoc payloadEnvelopeDoc);

		/// Parses the PayloadEnvelopeDoc from the string.
		SmartPtrCPayloadEnvelopeDoc PAYLOADENVELOPEXML_LINKAGE parsePayloadEnvelopeFromString(
			const std::string xml);

		/// Saves the PayloadEnvelopeDoc to a file.
		void PAYLOADENVELOPEXML_LINKAGE savePayloadEnvelopeToFile(
			const SmartPtrCPayloadEnvelopeDoc payloadEnvelopeDoc,
			const std::string filePath);

		/// Parses the PayloadEnvelopeDoc from the file.
		SmartPtrCPayloadEnvelopeDoc PAYLOADENVELOPEXML_LINKAGE parsePayloadEnvelopeFromFile(
			const std::string filePath);
	}
}

#endif /* PayloadEnvelopeXmlRoots_h_ */
