/*
 *  Created on: Aug 16, 2012
 *      Author: mdonahue
 *
 *  Copyright (C) 2012-2016 VMware, Inc.  All rights reserved. -- VMware Confidential
 */

#ifndef commIntegrationDefines_h
#define commIntegrationDefines_h

namespace Caf {

	extern const char* _sObjIdCommIntegrationCmsMessageTransformer;
	extern const char* _sObjIdCommIntegrationCmsMessageTransformerInstance;
	extern const char* _sObjIdCommIntegrationEventTopicCalculator;
	extern const char* _sObjIdCommIntegrationIncomingMessageHandler;
	extern const char* _sObjIdCommIntegrationIncomingMessageHandlerInstance;
	extern const char* _sObjIdCommIntegrationOutgoingMessageHandler;
	extern const char* _sObjIdCommIntegrationProtocolHeaderEnricher;
	extern const char* _sObjIdCommIntegrationProtocolHeaderEnricherInstance;
	extern const char* _sObjIdCommIntegrationReplyToCacher;
	extern const char* _sObjIdCommIntegrationReplyToCacherInstance;
	extern const char* _sObjIdCommIntegrationReplyToResolver;
}

#endif /* commIntegrationDefines_h */
