/*
 *  Author: bwilliams
 *  Created: April 6, 2012
 *
 *  Copyright (C) 2012-2016 VMware, Inc.  All rights reserved. -- VMware Confidential
 *
 *  This code was generated by the script "build/dev/codeGen/genCppXml". Please
 *  speak to Brian W. before modifying it by hand.
 *
 */

#ifndef ProviderResultsXmlRoots_h_
#define ProviderResultsXmlRoots_h_


#include "Doc/ProviderResultsDoc/CCdifDoc.h"

#include "Doc/DocXml/ProviderResultsXml/ProviderResultsXmlLink.h"
#include "Doc/ProviderResultsDoc/CSchemaDoc.h"

namespace Caf {

	namespace XmlRoots {

		/// Saves the SchemaDoc to a string.
		std::string PROVIDERRESULTSXML_LINKAGE saveSchemaToString(
			const SmartPtrCSchemaDoc schemaDoc);

		/// Parses the DefinitionObjectCollectionDoc from the string.
		SmartPtrCSchemaDoc PROVIDERRESULTSXML_LINKAGE parseSchemaFromString(
			const std::string xml);

		/// Saves the SchemaDoc to a file.
		void PROVIDERRESULTSXML_LINKAGE saveSchemaToFile(
			const SmartPtrCSchemaDoc schemaDoc,
			const std::string filePath);

		/// Parses the DefinitionObjectCollectionDoc from the file.
		SmartPtrCSchemaDoc PROVIDERRESULTSXML_LINKAGE parseSchemaFromFile(
			const std::string filePath);

		/// Saves the CdifDoc to a string.
		std::string PROVIDERRESULTSXML_LINKAGE saveCdifToString(
			const SmartPtrCCdifDoc cdifDoc);

		/// Parses the DefinitionObjectCollectionDoc from the string.
		SmartPtrCCdifDoc PROVIDERRESULTSXML_LINKAGE parseCdifFromString(
			const std::string xml);

		/// Saves the CdifDoc to a file.
		void PROVIDERRESULTSXML_LINKAGE saveCdifToFile(
			const SmartPtrCCdifDoc cdifDoc,
			const std::string filePath);

		/// Parses the DefinitionObjectCollectionDoc from the file.
		SmartPtrCCdifDoc PROVIDERRESULTSXML_LINKAGE parseCdifFromFile(
			const std::string filePath);
	}
}

#endif /* ProviderResultsXmlRoots_h_ */
