/*
 *	Copyright (C) 2011-2016 VMware, Inc.  All rights reserved. -- VMware Confidential
 */

#include "stdafx.h"

#ifdef WIN32

extern "C"
{

BOOL APIENTRY DllMain(HMODULE, uint32, LPVOID)
{
    return TRUE;
}

}

#endif
