/*
 *	 Author: bwilliams
 *  Created: April 6, 2012
 *
 *	Copyright (C) 2012-2016 VMware, Inc.  All rights reserved. -- VMware Confidential
 */

#ifndef DocContracts_h_
#define DocContracts_h_

#include "Doc/CafCoreTypesDoc/CafCoreTypesDocLink.h"
#include "Doc/SchemaTypesDoc/SchemaTypesDocLink.h"
#include "Doc/DiagTypesDoc/DiagTypesDocLink.h"
#include "Doc/DiagRequestDoc/DiagRequestDocLink.h"
#include "Doc/MgmtTypesDoc/MgmtTypesDocLink.h"
#include "Doc/MgmtRequestDoc/MgmtRequestDocLink.h"
#include "Doc/PayloadEnvelopeDoc/PayloadEnvelopeDocLink.h"
#include "Doc/PersistenceDoc/PersistenceDocLink.h"
#include "Doc/ProviderInfraDoc/ProviderInfraDocLink.h"
#include "Doc/ProviderRequestDoc/ProviderRequestDocLink.h"
#include "Doc/ProviderResultsDoc/ProviderResultsDocLink.h"
#include "Doc/ResponseDoc/ResponseDocLink.h"
#include "Doc/CafInstallRequestDoc/CafInstallRequestDocLink.h"
#include "Doc/MultiPmeMgmtRequestDoc/MultiPmeMgmtRequestDocLink.h"

#endif
