/*
 *  Created on: Aug 20, 2012
 *      Author: mdonahue
 *
 *  Copyright (C) 2012-2016 VMware, Inc.  All rights reserved. -- VMware Confidential
 */

#ifndef stdafx_h
#define stdafx_h

#include <CommonDefines.h>
#include <Integration.h>

#ifdef WIN32
#include "CWinService.h"
#endif


#endif /* stdafx_h */
