/*
 *  Author: bwilliams
 *  Created: April 6, 2012
 *
 *  Copyright (C) 2012-2016 VMware, Inc.  All rights reserved. -- VMware Confidential
 *
 *  This code was generated by the script "build/dev/codeGen/genCppXml". Please
 *  speak to Brian W. before modifying it by hand.
 *
 */

#ifndef ProviderRequestXmlRoots_h_
#define ProviderRequestXmlRoots_h_


#include "Doc/ProviderRequestDoc/CProviderCollectSchemaRequestDoc.h"

#include "Doc/DocXml/ProviderRequestXml/ProviderRequestXmlLink.h"
#include "Doc/ProviderRequestDoc/CProviderRequestDoc.h"

namespace Caf {

	namespace XmlRoots {

		/// Saves the ProviderCollectSchemaRequestDoc to a string.
		std::string PROVIDERREQUESTXML_LINKAGE saveProviderCollectSchemaRequestToString(
			const SmartPtrCProviderCollectSchemaRequestDoc providerCollectSchemaRequestDoc);

		/// Parses the ProviderCollectSchemaRequestDoc from the string.
		SmartPtrCProviderCollectSchemaRequestDoc PROVIDERREQUESTXML_LINKAGE parseProviderCollectSchemaRequestFromString(
			const std::string xml);

		/// Saves the ProviderCollectSchemaRequestDoc to a file.
		void PROVIDERREQUESTXML_LINKAGE saveProviderCollectSchemaRequestToFile(
			const SmartPtrCProviderCollectSchemaRequestDoc providerCollectSchemaRequestDoc,
			const std::string filePath);

		/// Parses the ProviderCollectSchemaRequestDoc from the file.
		SmartPtrCProviderCollectSchemaRequestDoc PROVIDERREQUESTXML_LINKAGE parseProviderCollectSchemaRequestFromFile(
			const std::string filePath);

		/// Saves the ProviderRequestDoc to a string.
		std::string PROVIDERREQUESTXML_LINKAGE saveProviderRequestToString(
			const SmartPtrCProviderRequestDoc providerRequestDoc);

		/// Parses the ProviderCollectSchemaRequestDoc from the string.
		SmartPtrCProviderRequestDoc PROVIDERREQUESTXML_LINKAGE parseProviderRequestFromString(
			const std::string xml);

		/// Saves the ProviderRequestDoc to a file.
		void PROVIDERREQUESTXML_LINKAGE saveProviderRequestToFile(
			const SmartPtrCProviderRequestDoc providerRequestDoc,
			const std::string filePath);

		/// Parses the ProviderCollectSchemaRequestDoc from the file.
		SmartPtrCProviderRequestDoc PROVIDERREQUESTXML_LINKAGE parseProviderRequestFromFile(
			const std::string filePath);
	}
}

#endif /* ProviderRequestXmlRoots_h_ */
