/*
 *	 Author: bwilliams
 *  Created: Oct 22, 2010
 *
 *	Copyright (C) 2010-2016 VMware, Inc.  All rights reserved. -- VMware Confidential
 */

#ifndef IntegrationSubsys_h_
#define IntegrationSubsys_h_

namespace Caf {
	// Integration Contracts
	extern const char* _sObjIdIntegrationObjectFactory;
	extern const char* _sObjIdErrorChannel;
	extern const char* _sObjIdNullChannel;
	extern const char* _sObjIdHeaderExpressionInvoker;
}

#endif // #ifndef IntegrationSubsys_h_
