/*
 *  Author: bwilliams
 *  Created: April 6, 2012
 *
 *  Copyright (C) 2012-2016 VMware, Inc.  All rights reserved. -- VMware Confidential
 *
 *  This code was generated by the script "build/dev/codeGen/genCppXml". Please
 *  speak to Brian W. before modifying it by hand.
 *
 */

#ifndef MultiPmeMgmtRequestXmlRoots_h_
#define MultiPmeMgmtRequestXmlRoots_h_


#include "Doc/MultiPmeMgmtRequestDoc/CMultiPmeMgmtRequestDoc.h"

#include "Doc/DocXml/MultiPmeMgmtRequestXml/MultiPmeMgmtRequestXmlLink.h"

namespace Caf {

	namespace XmlRoots {

		/// Saves the MultiPmeMgmtRequestDoc to a string.
		std::string MULTIPMEMGMTREQUESTXML_LINKAGE saveMultiPmeMgmtRequestToString(
			const SmartPtrCMultiPmeMgmtRequestDoc multiPmeMgmtRequestDoc);

		/// Parses the MultiPmeMgmtRequestDoc from the string.
		SmartPtrCMultiPmeMgmtRequestDoc MULTIPMEMGMTREQUESTXML_LINKAGE parseMultiPmeMgmtRequestFromString(
			const std::string xml);

		/// Saves the MultiPmeMgmtRequestDoc to a file.
		void MULTIPMEMGMTREQUESTXML_LINKAGE saveMultiPmeMgmtRequestToFile(
			const SmartPtrCMultiPmeMgmtRequestDoc multiPmeMgmtRequestDoc,
			const std::string filePath);

		/// Parses the MultiPmeMgmtRequestDoc from the file.
		SmartPtrCMultiPmeMgmtRequestDoc MULTIPMEMGMTREQUESTXML_LINKAGE parseMultiPmeMgmtRequestFromFile(
			const std::string filePath);
	}
}

#endif /* MultiPmeMgmtRequestXmlRoots_h_ */
