/*
 *  Created on: Aug 16, 2012
 *      Author: mdonahue
 *
 *  Copyright (C) 2012-2016 VMware, Inc.  All rights reserved. -- VMware Confidential
 */

#ifndef stdafx_h
#define stdafx_h

#include <CommonDefines.h>
#include <Integration.h>
#include "../../../amqpCore/src/amqpCore/amqpIntegrationCoreLink.h"
#include "commIntegrationDefines.h"

#include <DocUtils.h>
#include "CMessagePartsParser.h"
#include "CMessagePartsBuilder.h"
#include "CMessagePartDescriptorCalculator.h"
#include "CCmsMessageUtils.h"

#endif /* stdafx_h */
