/*
 *	 Author: bwilliams
 *  Created: 10/19/2010
 *
 *	Copyright (C) 2010-2016 VMware, Inc.  All rights reserved. -- VMware Confidential
 */

#include "stdafx.h"
#include "Integration/Core/FileHeaders.h"

namespace Caf {
	namespace FileHeaders {
		const char* _sFILENAME = "file_name";
		const char* _sORIGINAL_FILE = "file_originalFile";
	}
}
