#define BUILD_NUMBER \
	"build-12406962"
#define BUILD_NUMBER_NUMERIC \
	12406962
#define BUILD_NUMBER_NUMERIC_STRING \
	"12406962"
#define PRODUCT_BUILD_NUMBER \
	"product-build-10540"
#define PRODUCT_BUILD_NUMBER_NUMERIC \
	10540
#define PRODUCT_BUILD_NUMBER_NUMERIC_STRING \
	"10540"
