/*
 *  Created: Mar 11, 2004
 *
 *	Copyright (C) 2004-2016 VMware, Inc.  All rights reserved. -- VMware Confidential
 */

#ifndef _SubSystemBaseInc_h_
#define _SubSystemBaseInc_h_

#include "TCafSubSystemObjectRoot.h"
#include "TCafSubSystemSmartCl.h"

#endif // #ifndef _SubSystemBaseInc_h_
