/*
 *	Author: bwilliams
 *  Created: Oct 20, 2011
 *
 *	Copyright (C) 2011-2016 VMware, Inc.  All rights reserved. -- VMware Confidential
 */

#ifndef COMMON_SYS_INC_DOCXML_H_
#define COMMON_SYS_INC_DOCXML_H_

#include <CommonDefines.h>
#include <DocContracts.h>

#include "Doc/DocXml/CafCoreTypesXml/CafCoreTypesXmlLink.h"
#include "Doc/DocXml/SchemaTypesXml/SchemaTypesXmlLink.h"
#include "Doc/DocXml/DiagTypesXml/DiagTypesXmlLink.h"
#include "Doc/DocXml/DiagRequestXml/DiagRequestXmlLink.h"
#include "Doc/DocXml/MgmtTypesXml/MgmtTypesXmlLink.h"
#include "Doc/DocXml/MgmtRequestXml/MgmtRequestXmlLink.h"
#include "Doc/DocXml/PayloadEnvelopeXml/PayloadEnvelopeXmlLink.h"
#include "Doc/DocXml/PersistenceXml/PersistenceXmlLink.h"
#include "Doc/DocXml/ProviderInfraXml/ProviderInfraXmlLink.h"
#include "Doc/DocXml/ProviderRequestXml/ProviderRequestXmlLink.h"
#include "Doc/DocXml/ProviderResultsXml/ProviderResultsXmlLink.h"
#include "Doc/DocXml/ResponseXml/ResponseXmlLink.h"
#include "Doc/DocXml/CafInstallRequestXml/CafInstallRequestXmlLink.h"
#include "Doc/DocXml/MgmtRequestXml/MgmtRequestXmlLink.h"

#endif /* COMMON_SYS_INC_DOCXML_H_ */
