#!/bin/sh

export CAF_TMP_DIR=$CAF_OUTPUT_DIR/tmp
export CAF_INPUT_DIR=$CAF_INPUT_DIR
export CAF_OUTPUT_DIR=$CAF_OUTPUT_DIR
export CAF_PROVIDERREG_DIR=$CAF_OUTPUT_DIR/providerReg
export CAF_INVOKERS_DIR=$CAF_OUTPUT_DIR/invokers
export CAF_PROVIDERS_DIR=$CAF_OUTPUT_DIR/providers
export CAF_COMMONPACKAGES_DIR=$CAF_OUTPUT_DIR/commonPackages
