/*
 *	Author: bwilliams
 *  Created: Oct 20, 2011
 *
 *	Copyright (C) 2011-2016 VMware, Inc.  All rights reserved. -- VMware Confidential
 */

#ifndef FileHeadersInc_h_
#define FileHeadersInc_h_

namespace Caf {
	namespace FileHeaders {
		extern INTEGRATIONCORE_LINKAGE const char* _sFILENAME;
		extern INTEGRATIONCORE_LINKAGE const char* _sORIGINAL_FILE;
	}
}

#endif // #ifndef FileHeadersInc_h_

