/*
 *  Author: bwilliams
 *  Created: April 6, 2012
 *
 *  Copyright (C) 2012-2016 VMware, Inc.  All rights reserved. -- VMware Confidential
 *
 *  This code was generated by the script "build/dev/codeGen/genCppXml". Please
 *  speak to Brian W. before modifying it by hand.
 *
 */

#ifndef RequestInstanceParameterXml_h_
#define RequestInstanceParameterXml_h_


#include "Doc/CafCoreTypesDoc/CRequestInstanceParameterDoc.h"

#include "Doc/DocXml/CafCoreTypesXml/CafCoreTypesXmlLink.h"
#include "Xml/XmlUtils/CXmlElement.h"

namespace Caf {

	/// Streams the RequestInstanceParameter class to/from XML
	namespace RequestInstanceParameterXml {

		/// Adds the RequestInstanceParameterDoc into the XML.
		void CAFCORETYPESXML_LINKAGE add(
			const SmartPtrCRequestInstanceParameterDoc requestInstanceParameterDoc,
			const SmartPtrCXmlElement thisXml);

		/// Parses the RequestInstanceParameterDoc from the XML.
		SmartPtrCRequestInstanceParameterDoc CAFCORETYPESXML_LINKAGE parse(
			const SmartPtrCXmlElement thisXml);
	}
}

#endif
