/*
 *  Created on: May 10, 2012
 *      Author: mdonahue
 *
 *	Copyright (C) 2012-2016 VMware, Inc.  All rights reserved. -- VMware Confidential
 */

#ifndef AMQPIMPL_STDAFX_H_
#define AMQPIMPL_STDAFX_H_

#include "../stdafx.h"
#include "../AMQUtil.h"
#include "amqpImplInt.h"

#endif
