/*
 *  Author: bwilliams
 *  Created: April 6, 2012
 *
 *  Copyright (C) 2012-2016 VMware, Inc.  All rights reserved. -- VMware Confidential
 *
 *  This code was generated by the script "build/dev/codeGen/genCppXml". Please
 *  speak to Brian W. before modifying it by hand.
 *
 */

#ifndef ProviderResultsXml_Link_h_
#define ProviderResultsXml_Link_h_

#ifndef PROVIDERRESULTSXML_LINKAGE
    #ifdef WIN32
        #ifdef FRAMEWORK_BUILD
            #define PROVIDERRESULTSXML_LINKAGE __declspec(dllexport)
        #else
            #define PROVIDERRESULTSXML_LINKAGE __declspec(dllimport)
        #endif
    #else
        #define PROVIDERRESULTSXML_LINKAGE
    #endif
#endif


#endif /* ProviderResultsXml_Link_h_ */
