/*
 *	 Author: bwilliams
 *  Created: Oct 22, 2010
 *
 *	Copyright (C) 2010-2016 VMware, Inc.  All rights reserved. -- VMware Confidential
 */

#ifndef stdafx_h_
#define stdafx_h_

#include <CommonDefines.h>
#include <Integration.h>

#ifdef WIN32
#include "CWinService.h"
#endif


#endif // #ifndef stdafx_h_
