/*
 *	Author: bwilliams
 *  Created: Oct 20, 2011
 *
 *	Copyright (C) 2011-2016 VMware, Inc.  All rights reserved. -- VMware Confidential
 */

#ifndef COMMON_SYS_INC_DOCUTILS_H_
#define COMMON_SYS_INC_DOCUTILS_H_

#include <CommonDefines.h>
#include <DocContracts.h>

#include "../src/Doc/DocUtils/DocUtilsLink.h"

#endif /* COMMON_SYS_INC_DOCUTILS_H_ */
