/*
 *	 Author: mdonahue
 *  Created: Jan 13, 2011
 *
 *	Copyright (C) 2011-2016 VMware, Inc.  All rights reserved. -- VMware Confidential
 */

#ifndef PLATFORMSTRINGFUNC_H_
#define PLATFORMSTRINGFUNC_H_

namespace BasePlatform {

	//extern std::wstring A2W(const std::string& str);
	//extern std::string W2A(const std::wstring& str);
}

#endif /* PLATFORMSTRINGFUNC_H_ */
