/*
 *	 Author: bwilliams
 *  Created: 1/20/2010
 *
 *	Copyright (C) 2010-2016 VMware, Inc.  All rights reserved. -- VMware Confidential
 */

#ifndef MaContracts_h_
#define MaContracts_h_

//#include <DocContracts.h>

// Interfaces

#endif
