/*
 *	Author: bwilliams
 *  Created: Oct 20, 2011
 *
 *	Copyright (C) 2011-2016 VMware, Inc.  All rights reserved. -- VMware Confidential
 */

#ifndef COMMON_SYS_INC_INTEGRATION_H_
#define COMMON_SYS_INC_INTEGRATION_H_

#include <CommonDefines.h>

// Dependencies

// Interfaces
#include "Integration/IThrowable.h"

#include "Integration/Caf/IntegrationCafLink.h"
#include "Integration/Core/IntegrationCoreLink.h"

#endif /* COMMON_SYS_INC_INTEGRATION_H_ */
