/*
 *	 Author: bwilliams
 *  Created: Oct 22, 2010
 *
 *	Copyright (C) 2010-2016 VMware, Inc.  All rights reserved. -- VMware Confidential
 */

#include "stdafx.h"

extern "C" {

BOOL APIENTRY DllMain(HMODULE hModule, uint32 dwReason, LPVOID lpReserved) {
	return TRUE;
}

}
