/*
 *	 Author: bwilliams
 *  Created: Oct 22, 2010
 *
 *	Copyright (C) 2010-2016 VMware, Inc.  All rights reserved. -- VMware Confidential
 */

#ifndef CafIntegrationSubsys_h_
#define CafIntegrationSubsys_h_

namespace Caf {
	extern const char* _sObjIdErrorToResponseTransformerInstance;
	extern const char* _sObjIdErrorToResponseTransformer;
	extern const char* _sObjIdPayloadHeaderEnricherInstance;
	extern const char* _sObjIdPayloadHeaderEnricher;
	extern const char* _sObjIdEnvelopeToPayloadTransformerInstance;
	extern const char* _sObjIdEnvelopeToPayloadTransformer;
}

#endif // #ifndef CafIntegrationSubsys_h_
