/*
 *  Author: bwilliams
 *  Created: Nov 21, 2014
 *
 *  Copyright (C) 2012-2016 VMware, Inc.  All rights reserved. -- VMware Confidential
 *
 *  This code was generated by the script "build/dev/codeGen/genCppXml". Please
 *  speak to Brian W. before modifying it by hand.
 *
 */

#include "stdafx.h"

#include "Xml/XmlUtils/CXmlElement.h"
#include "Doc/DocXml/CafCoreTypesXml/RequestXml.h"

using namespace Caf;

SmartPtrCXmlElement RequestXml::parseString(
	const std::string& xml,
	const std::string& rootName) {
	CAF_CM_STATIC_FUNC_VALIDATE("RequestXml", "parseString");
	CAF_CM_VALIDATE_STRING(xml);
	// rootName is optional

	SmartPtrCXmlElement rootXml = CXmlUtils::parseString(xml, rootName);

	return rootXml;
}

SmartPtrCXmlElement RequestXml::parseFile(
	const std::string& xml,
	const std::string& rootName) {
	CAF_CM_STATIC_FUNC_VALIDATE("RequestXml", "parseFile");
	CAF_CM_VALIDATE_STRING(xml);
	// rootName is optional

	SmartPtrCXmlElement rootXml = CXmlUtils::parseFile(xml, rootName);

	return rootXml;
}
