/*
 *  Author: bwilliams
 *  Created: April 6, 2012
 *
 *  Copyright (C) 2012-2016 VMware, Inc.  All rights reserved. -- VMware Confidential
 *
 *  This code was generated by the script "build/dev/codeGen/genCppXml". Please
 *  speak to Brian W. before modifying it by hand.
 *
 */

#ifndef STDAFX_H_
#define STDAFX_H_

#ifdef WIN32
	#define MULTIPMEMGMTREQUESTXML_LINKAGE __declspec(dllexport)
#else
	#define MULTIPMEMGMTREQUESTXML_LINKAGE
#endif

#include <CommonDefines.h>

#include <DocContracts.h>
#include <DocUtils.h>


#endif /* STDAFX_H_ */
